#include "allegro.h"

#include "GfxSoundConfig.h"
#include "GameSettings.h"

int gfxToMain(void);
int gfx_my_button_proc (int, DIALOG *, int);
int pickScreenMode(void);
int pickDefaultScreenMode(void);
int tryToSwitchMode (int, int, int, int);
int screenModeFilter (int, int, int, int);
void gfxRedrawMenu ();

char gfxDescription [80] = "";
GameSettings * gfxgs = NULL;

DIALOG gfxConfigDialog [] = 
  {
    /* (dialog proc) (x) (y) (w) (h) (fg) (bg) (key) (flags) (d1) (d2) (dp) (dp2) (dp3) */
    { d_clear_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL },
    { d_text_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, gfxDescription, NULL, NULL },
    { gfx_my_button_proc, 0, 20, 200, 20, 0, 0,  0, D_EXIT, 0, 0, (char *)"Change Graphics Mode", NULL, (void *)pickScreenMode },
    { gfx_my_button_proc, 0, 55, 200, 20, 0, 0,  0, D_EXIT, 0, 0, (char *)"Default", NULL, (void *)pickDefaultScreenMode },
    { gfx_my_button_proc, 0, 90, 200, 20, 0, 0, 0, D_EXIT, 0, 0, (char *)"Main Menu", NULL, (void *)gfxToMain },
    { d_yield_proc, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL},
    { NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL  }
  };

int gfx_my_button_proc (int msg, DIALOG *d, int c)
{
   int ret = d_button_proc (msg, d, c);
   if (ret == D_CLOSE && d->dp3)
      return ((int (*)(void))d->dp3)();
   return ret;
}

int gfxToMain(void)
{
  //save settings
  gfxgs -> saveGraphicsSettings ();

  return D_CLOSE;  
}

int pickScreenMode(void)
{
  int gfxCard2 = gfxgs -> getGfxDriver ();
  int gfxWidth2 = gfxgs -> getResX ();
  int gfxHeight2 = gfxgs -> getResY ();
  int gfxDepth2 = gfxgs -> getColorDepth ();
  int gfxSetResponse = 0;

  gfxSetResponse = gfx_mode_select_filter(&gfxCard2, &gfxWidth2, &gfxHeight2, &gfxDepth2, screenModeFilter);

  //if 0, user cancelled process
  if (gfxSetResponse != 0)
    {
      tryToSwitchMode (gfxDepth2, gfxCard2, gfxWidth2, gfxHeight2);
    }
  return D_O_K;
}

int pickDefaultScreenMode(void)
{
  int gfxCard2 = GFX_AUTODETECT_WINDOWED;
  int gfxWidth2 = 640;
  int gfxHeight2 = 480;
  int gfxDepth2 = 32;

  tryToSwitchMode (gfxDepth2, gfxCard2, gfxWidth2, gfxHeight2);

  //OK, it worked. Copy the variables over.
  //gfxCard = gfxCard2;
  //gfxWidth = gfxWidth2;
  //gfxHeight = gfxHeight2;
  //gfxDepth = gfxDepth2;
  
  sprintf (gfxDescription, "%dx%d at %d-bit color", gfxWidth2, gfxHeight2, gfxDepth2);

  gfxgs -> setGfxSettings (gfxDepth2, gfxCard2, gfxWidth2, gfxHeight2);
  return D_O_K;
}

int tryToSwitchMode (int gfxDepth2, int gfxCard2, int gfxWidth2, int gfxHeight2)
{
  int gfxSetResponse = 1;

  //try to set the mode and cancel if it fails
  set_color_depth (gfxDepth2);
  gfxSetResponse = set_gfx_mode(gfxCard2, gfxWidth2, gfxHeight2, 0, 0);
  if (gfxSetResponse != 0 || gfxDepth2 == 8)
    {
      //! error!
      set_color_depth (gfxgs -> getColorDepth ());
      set_gfx_mode(gfxgs -> getGfxDriver (), gfxgs -> getResX (), gfxgs -> getResY (), 0, 0);
      
      if (gfxDepth2 != 8)
	alert("These graphics settings don't work.", "Try a different set.", NULL, "&OK", NULL, 'o', 0);
      else
	alert("TASM does not support 8-bit color depth.", "Try a different color depth.", NULL, "&OK", NULL, 'o', 0);
      
      show_mouse (screen);
    }
  else
    {
      //OK, it worked. Copy the variables over.
      sprintf (gfxDescription, "%dx%d at %d-bit color", gfxWidth2, gfxHeight2, gfxDepth2);
      alert("New graphics mode set!", "v031: Restart the program to avoid", "Ugly boxes!", "&OK", NULL, 'o', 0);
      gfxgs -> setGfxSettings (gfxDepth2, gfxCard2, gfxWidth2, gfxHeight2);
      set_dialog_color (gfxConfigDialog, makecol(0, 0, 0), makecol (192, 192, 192));
      dialog_message (gfxConfigDialog, MSG_DRAW, 0, NULL);
      show_mouse (screen);
    }

  return D_O_K;
}

int screenModeFilter (int c, int w, int h, int d)
{
  //this is used by gfx_mode_select_filter. It will return 0 (and will be included in the screen mode selection if:
  //it's not 8-bit color Depth. (no palettes!)
  //it's at least 640 pixels wide and 480 pixels tall
  if (d == 8 || w < 640 || h < 480)
    return 1;

  return 0;
}

void gfxRedrawMenu ()
{
  
}

GfxSoundConfig :: GfxSoundConfig (GameSettings * gs)
{
  gfxgs = gs;
  sprintf (gfxDescription, "%dx%d at %d-bit color", gfxgs -> getResX (), gfxgs -> getResY (), gfxgs -> getColorDepth ());
}

void GfxSoundConfig :: go (int index)
{
  centre_dialog (gfxConfigDialog);
  set_dialog_color (gfxConfigDialog, gui_fg_color, gui_bg_color);
  dialog_message (gfxConfigDialog, MSG_DRAW, 0, NULL);
  show_mouse (screen);
  do_dialog (gfxConfigDialog, index);
}

GfxSoundConfig :: ~GfxSoundConfig ()
{
}
