#include "allegro.h"

#include "UnitInput.h"
#include "gameConstants.h"
#include "Unit.h"
#include "gameConstants.h"
#include "tasmMath.h"

#include <vector>
#include <string>
using namespace std;

UnitInput :: UnitInput ()
{
  targetAffiliation = affiliationPlayer;
  walkTime = 5;
  walkWait = 5;

  //if the unit should home in on a target by move forward -> correct angle
  homeTargetAngleCorrection = true;
  homeTimeInbetweenCorrection = 3;
  homeAngleCorrection = itofix (15);

  //attack periodically
  attackPeriodically = false;
  attackPeriod = 30;
  attackPeriodIndex = 0;

  //counters
  setCounters ();
}

UnitInput :: UnitInput (UnitInput * utCopy)
{
  //w00t! Copy constructor!
  walkTime = utCopy -> walkTime;
  walkWait = utCopy -> walkWait;
  homeTargetAngleCorrection = utCopy -> homeTargetAngleCorrection;
  homeTimeInbetweenCorrection = utCopy -> homeTimeInbetweenCorrection;
  homeAngleCorrection = utCopy -> homeAngleCorrection;
  targetAffiliation = utCopy -> targetAffiliation;
  attackPeriodically = utCopy -> attackPeriodically;
  attackPeriod = utCopy -> attackPeriod;
  attackPeriodIndex = utCopy -> attackPeriodIndex;

  setCounters ();
}

void UnitInput :: setCounters ()
{
  walkingDir = itofix (0);
  targetLocated = false;
  targetUnit = NULL;
  walkingMode = true;
  walkTimeCounter = 0;
  walkWaitCounter = 0;
  homeTimeInbetweenCorrectionCounter = 0;
  attackPeriodCounter = 0;  
}

void UnitInput :: decideAction (Unit * thisUnit, UnitList * units)
{
  //decides the action to take this round.

  //make sure the unit is active. (not dead or stunned)
  int cA = thisUnit -> getCharAnim ();
  if (cA == charAnimDead || cA == charAnimStun)
    {
      return;
    }

  if (!units -> thisUnitExists (targetUnit))
    {
      locateTarget (thisUnit, units);
    }

  walkAndWait (thisUnit, units);

  if (targetLocated)
    {
      if (homeTargetAngleCorrection)
	homeTargetCorrect (thisUnit, units);

      if (attackPeriodically)
	attackP (thisUnit, units);
    }
}

void UnitInput :: homeTargetCorrect (Unit * thisUnit, UnitList * units)
{
  fixed diffAngle;
  
  homeTimeInbetweenCorrectionCounter ++;
  
  if (homeTimeInbetweenCorrectionCounter > homeTimeInbetweenCorrection)
    {
      //get the difference in angle between the direction the units are pointing and the direction it is facing
	  
      diffAngle = angleDist (calcAngle (thisUnit -> getPosX (), thisUnit -> getPosY (), targetUnit -> getPosX (), targetUnit -> getPosY (), walkingDir), thisUnit -> getWalkAngle ());
	  
      walkingDir = thisUnit -> getWalkAngle ();
	  
      if (diffAngle < (-1 * homeAngleCorrection))
	walkingDir -= homeAngleCorrection;
      else if (diffAngle > homeAngleCorrection)
	walkingDir += homeAngleCorrection;
      
      thisUnit -> faceAngle (walkingDir);
	  
      homeTimeInbetweenCorrectionCounter = 0;
    }
}

void UnitInput :: walkAndWait (Unit * thisUnit, UnitList * units)
{
  //this alternates between walking and waiting.
  //to always walk, set walkWait to 0.
  //to always wait, set walkTime to 0.

  thisUnit -> walk (walkingMode);

  if (walkingMode)
    {
      walkTimeCounter++;
      if (walkTimeCounter >= walkTime && walkWait > 0)
	{
	  walkingMode = !walkingMode;
	  walkTimeCounter = 0;
	}
    }
  else
    {
      walkWaitCounter++;
      if (walkWaitCounter >= walkWait && walkTime > 0)
	{
	  walkingMode = !walkingMode;
	  walkWaitCounter = 0;
	}
    }
}

void UnitInput :: locateTarget (Unit * thisUnit, UnitList * units)
{
  vector <Unit *> * targetAffiliationUnits;

  targetAffiliationUnits = units -> search (searchByAffiliation, targetAffiliation);

  if (targetAffiliationUnits -> size () > 0)
    {
      //demo: pick a random one to go after!!!
      targetUnit = targetAffiliationUnits -> at (rand () % targetAffiliationUnits -> size ());
      targetLocated = true;
    }
}

void UnitInput :: attackP (Unit * thisUnit, UnitList * units)
{
  attackPeriodCounter ++;

  if (int(thisUnit -> getAddedUnits () -> size ()) > attackPeriodIndex && 
      attackPeriodCounter >= attackPeriod && 
      thisUnit -> canAddUnit (thisUnit -> getAddedUnits () -> at (attackPeriodIndex))
      )
    {
      attackPeriodCounter = 0;
      thisUnit -> addUnit (thisUnit -> getAddedUnits () -> at (attackPeriodIndex));
    }
}

UnitInput :: ~UnitInput ()
{
}
